<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 30/10/14
 * Time: 11:45
 */

namespace Folha\Logic\ESocial\Importacao;

use Folha\Entities\GpeEsocial;
use Folha\Entities\GpeEsocialRegistro;
use Folha\Entities\GpeEsocialResumo;
use Geral\Util\Ambiente;
use HCM\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Setforms2\Shared\Registry;

class Persistencia
{
    private $eventos;
    private $resumos;
    private $registros;

    private $nrorgpadrao;

    /**
     * @param $caminho          Pasta na qual se encontram os arquivos de leiaute
     * @param $versao           Versão do E-Social
     * @param $arq_eventos      Nome do arquivo com a descrição dos eventos
     * @param $arq_resumos      Nome do arquivo com os resumos dos eventos
     * @param $arq_registros    Nome dos arquivos com os registros do eventos
     * @throws \Exception
     */
    public function __construct($leiautes)
    {
        try {
            $this->validaNrEventos($leiautes);

            $this->eventos = $leiautes['EVENTOS'];
            $this->resumos = $leiautes['RESUMOS'];
            $this->registros = $leiautes['REGISTROS'];

            $this->nrorgpadrao = 0;

        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    public function persisteTabelas($versao)
    {
        try {
            Registry::getEntityManager()->beginTransaction();

            self::insereGpeEsocial($versao);
            //self::geraArvoreRegistro($versao);

            Registry::getEntityManager()->commit();
        } catch (\Exception $e) {
            Registry::getEntityManager()->rollback();
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * Validada se a quantidade de eventos em cada leiaute está consistente.
     *
     * @param $leiautes
     *
     * @return bool
     * @throws \Exception
     */
    private function validaNrEventos($leiautes)
    {
        try {
            $nrEventos = count($leiautes['EVENTOS']);
            $nrResumos = count($leiautes['RESUMOS']);
            $nrRegistros = count($leiautes['REGISTROS']);

            if (($nrEventos != $nrResumos) or ($nrResumos != $nrRegistros) or ($nrEventos != $nrRegistros)) {
                throw Exception::quantidadeEventos();
            }
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    private function insereGpeEsocial($versao)
    {
        try {
            foreach ($this->eventos as $evento) {

                $gpeEsocialOld = GpeEsocial::findOneByNomeAndVersao($evento['NOME'],$versao);
                if (is_object($gpeEsocialOld)) {
                    throw Exception::existeVersaoEvento();
                }

                $nrEsocial = NovoCodigo::geraCodigo($this->nrorgpadrao, 'GPE_ESOCIAL', 1, 22, true, false);

                self::insereGpeEsocialResumo($nrEsocial, $this->resumos[$evento['NOME']]);
                self::insereGpeEsocialRegistro($nrEsocial, $this->registros[$evento['NOME']]);

                $gpeEsocial = new GpeEsocial();
                $gpeEsocial->setNresocial($nrEsocial);
                $gpeEsocial->setDescricao($evento['DESCRICAO']);
                $gpeEsocial->setNome($evento['NOME']);
                $gpeEsocial->setIdgrupevent($evento['IDGRUPEVENT']);
                $gpeEsocial->setVersao($versao);
                $gpeEsocial->setNrorg($this->nrorgpadrao);
                $gpeEsocial->setDtinclusao(DateUtil::getDataAtual(false));
                $gpeEsocial->setCdoperinclusao(Ambiente::getCdOperador());
                $gpeEsocial->setNrorginclusao($this->nrorgpadrao);
                $gpeEsocial->setDtultatu(DateUtil::getDataAtual(false));
                $gpeEsocial->setCdoperultatu(Ambiente::getCdOperador());
                $gpeEsocial->setNrorgultatu($this->nrorgpadrao);
                $gpeEsocial->setIdativo('S');
                $gpeEsocial->persist();
            }

            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    private function insereGpeEsocialResumo($nrEsocial, $resumos)
    {
        try {
            foreach ($resumos as $resumo) {
                $nrResumo = NovoCodigo::geraCodigo($this->nrorgpadrao, 'GPE_ESOCIAL_RESUMO', 1, 22, true, false);

                $gpeEsocialResumo = new GpeEsocialResumo();
                $gpeEsocialResumo->setNrresumo($nrResumo);
                $gpeEsocialResumo->setNresocial($nrEsocial);
                $gpeEsocialResumo->setReg(preg_replace('~[\n]+~','',$resumo['REGISTRO']));
                $gpeEsocialResumo->setPai(preg_replace('~[\n]+~','',$resumo['PAI']));
                $gpeEsocialResumo->setNivel($resumo['NIVEL']);
                $gpeEsocialResumo->setDescricao(preg_replace('~[\n]+~',' ',$resumo['DESCRICAO']));
                $gpeEsocialResumo->setOcorrencia($resumo['OCORRENCIA']);
                $gpeEsocialResumo->setChave(preg_replace('~[\n]+~',' ',$resumo['CHAVE']));
                $gpeEsocialResumo->setCondicao(preg_replace('~[\n]+~',' ',$resumo['CONDICAO']));
                $gpeEsocialResumo->setNrorg($this->nrorgpadrao);
                $gpeEsocialResumo->setDtinclusao(DateUtil::getDataAtual(false));
                $gpeEsocialResumo->setCdoperinclusao(Ambiente::getCdOperador());
                $gpeEsocialResumo->setNrorginclusao($this->nrorgpadrao);
                $gpeEsocialResumo->setDtultatu(DateUtil::getDataAtual(false));
                $gpeEsocialResumo->setCdoperultatu(Ambiente::getCdOperador());
                $gpeEsocialResumo->setNrorgultatu($this->nrorgpadrao);
                $gpeEsocialResumo->setIdativo('S');
                $gpeEsocialResumo->persist();
            }
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    private function insereGpeEsocialRegistro($nrEsocial, $registros)
    {
        try {
            foreach ($registros as $registro) {
                $nrRegistro = NovoCodigo::geraCodigo($this->nrorgpadrao, 'GPE_ESOCIAL_REGISTRO', 1, 22, true, false);

                $gpeEsocialRegistro = new GpeEsocialRegistro();
                $gpeEsocialRegistro->setNrregistro($nrRegistro);
                $gpeEsocialRegistro->setNresocial($nrEsocial);
                $gpeEsocialRegistro->setCampo(preg_replace('~[\n]+~','',$registro['CAMPO']));
                $gpeEsocialRegistro->setPai(preg_replace('~[\n]+~','',$registro['PAI']));
                $gpeEsocialRegistro->setElemento($registro['ELEMENTO']==="CG" ? "G" : $registro['ELEMENTO']);
                $gpeEsocialRegistro->setTipo($registro['TIPO']);
                $gpeEsocialRegistro->setOcorrencia($registro['OCORRENCIA']);
                $gpeEsocialRegistro->setTamanho($registro['TAMANHO']==="-" ? null : $registro['TAMANHO']);
                $gpeEsocialRegistro->setDec($registro['DECIMAL']==="-" ? null : $registro['DECIMAL']);
                $gpeEsocialRegistro->setDescricao(preg_replace('~[\n]+~',' ',$registro['DESCRICAO']));
                $gpeEsocialRegistro->setNrorg($this->nrorgpadrao);
                $gpeEsocialRegistro->setDtinclusao(DateUtil::getDataAtual(false));
                $gpeEsocialRegistro->setCdoperinclusao(Ambiente::getCdOperador());
                $gpeEsocialRegistro->setNrorginclusao($this->nrorgpadrao);
                $gpeEsocialRegistro->setDtultatu(DateUtil::getDataAtual(false));
                $gpeEsocialRegistro->setCdoperultatu(Ambiente::getCdOperador());
                $gpeEsocialRegistro->setNrorgultatu($this->nrorgpadrao);
                $gpeEsocialRegistro->setIdativo('S');
                $gpeEsocialRegistro->persist();
            }
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    //@todo Verificar possibilidade de organizar registros pais e filhos através da chave nrregistro criando um campo nrpai para gerar as tags do projects de forma correta
    //@todo pois quando há campos com o mesmo nome ocorrem erros que devem ser corrigidos manualmente.
    private function geraArvoreRegistro($versao)
    {
        $gpeEsocial = GpeEsocial::findByVersao($versao);

        foreach ($gpeEsocial as $evento) {
            $nrEsocial = $evento->getNresocial();

            $gpeEsocialRegistro = \Folha\Repositories\GpeEsocial::getRegistrosOrdenados($nrEsocial);

            foreach ($gpeEsocialRegistro as $registro) {
                if (is_null($registro->getPai())) {
                    $nrPai = $registro->getNrregistro();
                }
                else {
                    $registro->setNrregpai($nrPai);
                }
            }
        }
    }
}